package resize_fat::dir_entry;





my $DELETED_FLAG      = 0xe5;
my $VOLUME_LABEL_ATTR = 0x08;
my $VFAT_ATTR         = 0x0f;
my $DIRECTORY_ATTR    = 0x10;

1;

sub get_cluster($) {
    my ($entry) = @_;
    $entry->{first_cluster} + ($resize_fat::isFAT32 ? $entry->{first_cluster_high} * 65536 : 0);
}
sub set_cluster($$) {
    my ($entry, $val) = @_;
    $entry->{first_cluster} = $val & (1 << 16) - 1;
    $entry->{first_cluster_high} = $val >> 16 if $resize_fat::isFAT32;
}

sub is_directory($) {
    my ($entry) = @_;
    $entry->{attributes} & $DIRECTORY_ATTR && $entry->{name} !~ /^\.\.? / && !is_special_entry($entry);
}

sub is_volume($) {
    my ($entry) = @_;
    !is_special_entry($entry) && $entry->{attributes} & $VOLUME_LABEL_ATTR;
}

sub is_file($) {
    my ($entry) = @_;
    !is_special_entry($entry) && !is_directory($entry) && !is_volume($entry) && $entry->{length};
}


sub is_special_entry($) {
    my ($entry) = @_;
    my ($c) = unpack "C", $entry->{name};

    
    $c == 0 || $c == $DELETED_FLAG || $c == 0xF6 and return 1;

    $entry->{attributes} == $VFAT_ATTR and return 1;
    0;
}



sub remap {
    my ($entry) = @_;

    is_special_entry($entry) and return;

    my $cluster = get_cluster($entry);
    my $new_cluster = resize_fat::c_rewritten::fat_remap($cluster);

    

    $new_cluster == $cluster and return; 

    set_cluster($entry, $new_cluster);
    1;
}
