package resize_fat::directory;




use common qw(:system);
use resize_fat::dir_entry;
use resize_fat::io;


my $format = "a8 a3 C C C S7 I";
my @fields = (
    'name',
    'extension',
    'attributes',
    'is_upper_case_name',
    'creation_time_low',	
    'creation_time_high',
    'creation_date',
    'access_date',
    'first_cluster_high',	
    'time',
    'date',
    'first_cluster',
    'length',
);
my $psizeof_format = psizeof($format);

1;

sub entry_size { $psizeof_format }



sub traverse($$) {
    my ($directory, $f) = @_;

    for (my $i = 0;; $i++) {
	my $raw = \substr($directory, $i * $psizeof_format, $psizeof_format);

	
	$$raw =~ /^\0*$/ and return $directory;

	my $entry; @{$entry}{@fields} = unpack $format, $$raw;

	&$f($entry)
	    and	$$raw = pack $format, @{$entry}{@fields};
    }
    $directory;
}

sub traverse_all($$) {
    my ($fs, $f) = @_;

    my $traverse_all; $traverse_all = sub {
	my ($entry) = @_;

	&$f($entry);

        resize_fat::dir_entry::is_directory($entry)
	    and traverse(resize_fat::io::read_file($fs, resize_fat::dir_entry::get_cluster($entry)), $traverse_all);

	undef; 
    };

    my $directory = $resize_fat::isFAT32 ?
	resize_fat::io::read_file($fs, $fs->{fat32_root_dir_cluster}) :
	resize_fat::io::read($fs, $fs->{root_dir_offset}, $fs->{root_dir_size});
    traverse($directory, $traverse_all);
    undef $traverse_all; 
}




sub remap($$) {
    my ($fs, $directory) = @_;
    traverse($directory, \&resize_fat::dir_entry::remap);
}
